@@ -133,6 +133,12 @@ class NoSuchMetric(IndexerException):
                                            metric)
         self.metric = metric
 
+    def jsonify(self):
+        return {
+            "cause": "Metric not found",
+            "detail": self.metric,
+        }
+
 
 class NoSuchResource(IndexerException):
     """Error raised when a resource does not exist."""
@@ -361,7 +367,7 @@ class IndexerDriver(object):
         raise exceptions.NotImplementedError
 
     @staticmethod
-    def update_archive_policy(name, ap_items):
+    def update_archive_policy(name, ap_items, **kwargs):
         raise exceptions.NotImplementedError
 
     @staticmethod
