@@ -260,27 +260,46 @@ class ResourceClassMapper(object):
 class SQLAlchemyIndexer(indexer.IndexerDriver):
     _RESOURCE_TYPE_MANAGER = ResourceClassMapper()
 
+    @staticmethod
+    def _set_url_database(url, database):
+        if hasattr(url, "set"):
+            return url.set(database=database)
+        else:
+            url.database = database
+            return url
+
+    @staticmethod
+    def _set_url_drivername(url, drivername):
+        if hasattr(url, "set"):
+            return url.set(drivername=drivername)
+        else:
+            url.drivername = drivername
+            return url
+
     @classmethod
     def _create_new_database(cls, url):
         """Used by testing to create a new database."""
         purl = sqlalchemy_url.make_url(
             cls.dress_url(
                 url))
-        purl.database = purl.database + str(uuid.uuid4()).replace('-', '')
+        new_database = purl.database + str(uuid.uuid4()).replace('-', '')
+        purl = cls._set_url_database(purl, new_database)
         new_url = str(purl)
         sqlalchemy_utils.create_database(new_url)
         return new_url
 
-    @staticmethod
-    def dress_url(url):
+    @classmethod
+    def dress_url(cls, url):
         # If no explicit driver has been set, we default to pymysql
         if url.startswith("mysql://"):
             url = sqlalchemy_url.make_url(url)
-            url.drivername = "mysql+pymysql"
+            new_drivername = "mysql+pymysql"
+            url = cls._set_url_drivername(url, new_drivername)
             return str(url)
         if url.startswith("postgresql://"):
             url = sqlalchemy_url.make_url(url)
-            url.drivername = "postgresql+psycopg2"
+            new_drivername = "postgresql+psycopg2"
+            url = cls._set_url_drivername(url, new_drivername)
             return str(url)
         return url
 
@@ -393,68 +412,127 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         return resource_type
 
     def update_resource_type(self, name, add_attributes=None,
-                             del_attributes=None):
-        if not add_attributes and not del_attributes:
+                             del_attributes=None, update_attributes=None):
+        if not add_attributes and not del_attributes and not update_attributes:
             return
         add_attributes = add_attributes or []
         del_attributes = del_attributes or []
+        update_attributes = update_attributes or []
 
-        self._set_resource_type_state(name, "updating", "active")
+        with self.facade.independent_writer() as session:
+            engine = session.connection()
+            rt = self._get_resource_type(session, name)
 
-        try:
-            with self.facade.independent_writer() as session:
-                engine = session.connection()
-                rt = self._get_resource_type(session, name)
+            with self.facade.writer_connection() as connection:
+                ctx = migration.MigrationContext.configure(connection)
+                op = operations.Operations(ctx)
+                self.fill_null_attribute_values(engine, name, rt, session,
+                                                update_attributes)
 
-                with self.facade.writer_connection() as connection:
-                    ctx = migration.MigrationContext.configure(connection)
-                    op = operations.Operations(ctx)
-                    for table in [rt.tablename, '%s_history' % rt.tablename]:
-                        with op.batch_alter_table(table) as batch_op:
-                            for attr in del_attributes:
-                                batch_op.drop_column(attr)
-                            for attr in add_attributes:
-                                server_default = attr.for_filling(
-                                    engine.dialect)
-                                batch_op.add_column(sqlalchemy.Column(
-                                    attr.name, attr.satype,
-                                    nullable=not attr.required,
-                                    server_default=server_default))
+                self._set_resource_type_state(name, "updating", "active")
+                for table in [rt.tablename, '%s_history' % rt.tablename]:
+                    with op.batch_alter_table(table) as batch_op:
+                        for attr in del_attributes:
+                            LOG.debug("Dropping column [%s] from resource [%s]"
+                                      " and database table [%s]",
+                                      attr, name, table)
+                            batch_op.drop_column(attr)
+                        for attr in add_attributes:
+                            LOG.debug("Adding new column [%s] with type [%s], "
+                                      "nullable [%s] and default value [%s] "
+                                      "in resource [%s] and database "
+                                      "table [%s]", attr.name, attr.satype,
+                                      not attr.required,
+                                      getattr(attr, 'fill', None), name, table)
 
-                                # We have all rows filled now, we can remove
-                                # the server_default
-                                if server_default is not None:
-                                    batch_op.alter_column(
-                                        column_name=attr.name,
-                                        existing_type=attr.satype,
-                                        existing_server_default=server_default,
-                                        existing_nullable=not attr.required,
-                                        server_default=None)
+                            server_default = attr.for_filling(
+                                engine.dialect)
+                            batch_op.add_column(sqlalchemy.Column(
+                                attr.name, attr.satype,
+                                nullable=not attr.required,
+                                server_default=server_default))
 
-                rt.state = "active"
-                rt.updated_at = utils.utcnow()
-                rt.attributes.extend(add_attributes)
-                for attr in list(rt.attributes):
-                    if attr.name in del_attributes:
-                        rt.attributes.remove(attr)
-                # FIXME(sileht): yeah that's wierd but attributes is a custom
-                # json column and 'extend' doesn't trigger sql update, this
-                # enforce the update. I wonder if sqlalchemy provides something
-                # on column description side.
-                sqlalchemy.orm.attributes.flag_modified(rt, 'attributes')
+                            # We have all rows filled now, we can remove
+                            # the server_default
+                            if server_default is not None:
+                                LOG.debug("Removing default value [%s] from "
+                                          "column [%s] of resource [%s] and "
+                                          "database table [%s]",
+                                          getattr(attr, 'fill', None),
+                                          attr.name, name, table)
+                                batch_op.alter_column(
+                                    column_name=attr.name,
+                                    existing_type=attr.satype,
+                                    existing_server_default=server_default,
+                                    existing_nullable=not attr.required,
+                                    server_default=None)
 
-        except Exception:
-            # NOTE(sileht): We fail the DDL, we have no way to automatically
-            # recover, just set a particular state
-            # TODO(sileht): Create a repair REST endpoint that delete
-            # columns not existing in the database but in the resource type
-            # description. This will allow to pass wrong update_error to active
-            # state, that currently not possible.
-            self._set_resource_type_state(name, "updating_error")
-            raise
+                        for attr in update_attributes:
+                            LOG.debug("Updating column [%s] from old values "
+                                      "type [%s], nullable [%s], to new values"
+                                      " type [%s], nullable [%s] of resource "
+                                      "[%s] and database_table [%s]",
+                                      attr[1].name, attr[1].satype,
+                                      not attr[1].required, attr[0].satype,
+                                      not attr[0].required, name, table)
+                            batch_op.alter_column(
+                                column_name=attr[1].name,
+                                existing_type=attr[1].satype,
+                                existing_nullable=not attr[1].required,
+                                type_=attr[0].satype,
+                                nullable=not attr[0].required)
+
+            rt.state = "active"
+            rt.updated_at = utils.utcnow()
+            rt.attributes.extend(add_attributes)
+            update_attributes = list(map(lambda a: a[0],
+                                         update_attributes))
+            update_attributes_names = list(map(lambda a: a.name,
+                                               update_attributes))
+            for attr in list(rt.attributes):
+                if (attr.name in del_attributes or
+                        attr.name in update_attributes_names):
+                    rt.attributes.remove(attr)
+
+            rt.attributes.extend(update_attributes)
+            # FIXME(sileht): yeah that's wierd but attributes is a custom
+            # json column and 'extend' doesn't trigger sql update, this
+            # enforce the update. I wonder if sqlalchemy provides something
+            # on column description side.
+            LOG.debug("Updating resource [%s] setting attributes as [%s]",
+                      name, list(rt.attributes))
+            sqlalchemy.orm.attributes.flag_modified(rt, 'attributes')
 
         return rt
 
+    def fill_null_attribute_values(self, engine, name, rt, session,
+                                   update_attributes):
+        for table in [rt.tablename, '%s_history' % rt.tablename]:
+            for attr in update_attributes:
+                if (hasattr(attr[0], 'fill') and
+                        attr[0].fill is not None):
+                    mappers = self._resource_type_to_mappers(
+                        session, name)
+                    if table == rt.tablename:
+                        resource_cls = mappers["resource"]
+                    else:
+                        resource_cls = mappers["history"]
+                    cls_attr = attr[0].name
+                    f = QueryTransformer.build_filter(
+                        engine.dialect.name, resource_cls,
+                        {'=': {cls_attr: None}})
+                    q = session.query(resource_cls).filter(
+                        f).with_for_update()
+                    resources = q.all()
+                    if resources:
+                        LOG.debug("Null resources [%s] to be filled with [%s] "
+                                  "for resource-type [%s]", resources,
+                                  attr[0].fill, name)
+                        for resource in resources:
+                            if hasattr(resource, attr[0].name):
+                                setattr(resource, attr[0].name,
+                                        attr[0].fill)
+
     def get_resource_type(self, name):
         with self.facade.independent_reader() as session:
             return self._get_resource_type(session, name)
@@ -579,7 +657,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         with self.facade.independent_reader() as session:
             return session.query(ArchivePolicy).get(name)
 
-    def update_archive_policy(self, name, ap_items):
+    def update_archive_policy(self, name, ap_items, **kwargs):
         with self.facade.independent_writer() as session:
             ap = session.query(ArchivePolicy).get(name)
             if not ap:
@@ -597,6 +675,8 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                         % utils.timespan_total_seconds(c.granularity))
             # NOTE(gordc): ORM doesn't update JSON column unless new
             ap.definition = ap_items
+            if kwargs.get("back_window") is not None:
+                ap.back_window = kwargs.get("back_window")
             return ap
 
     def delete_archive_policy(self, name):
@@ -819,12 +899,29 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             return r
 
     @retry_on_deadlock
-    def update_resource(self, resource_type,
-                        resource_id, ended_at=_marker, metrics=_marker,
-                        append_metrics=False,
-                        create_revision=True,
+    def update_resource(self, resource_type, resource_id,
+                        ended_at=_marker, metrics=_marker,
+                        append_metrics=False, create_revision=True,
                         **kwargs):
         with self.facade.writer() as session:
+            data_to_update = kwargs.copy()
+
+            data_to_update['ended_at'] = ended_at
+            data_to_update['metrics'] = metrics
+            if create_revision:
+                resource = self.get_resource(
+                    resource_type, resource_id, with_metrics=True)
+                if not utils.is_resource_revision_needed(
+                        resource, data_to_update):
+                    LOG.info("We thought that a revision for resource "
+                             "[%s] was needed. However, after locking the "
+                             "table and checking it again, we found that it "
+                             "is not needed anymore. This is due to a "
+                             "concurrency issue that might happen. Therefore, "
+                             "no revision is going to be generated at this "
+                             "time.", data_to_update)
+                    create_revision = False
+
             mappers = self._resource_type_to_mappers(session, resource_type)
             resource_cls = mappers["resource"]
             resource_history_cls = mappers["history"]
@@ -880,7 +977,9 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
                 session.flush()
             except exception.DBConstraintError as e:
-                if e.check_name == "ck_started_before_ended":
+                if e.check_name in (
+                        "ck_resource_started_before_ended",
+                        "ck_resource_history_started_before_ended"):
                     raise indexer.ResourceValueError(
                         resource_type, "ended_at", ended_at)
                 raise
@@ -949,8 +1048,6 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             target_cls = self._resource_type_to_mappers(
                 session, resource_type)["resource"]
 
-            q = session.query(target_cls.id)
-
             engine = session.connection()
             try:
                 f = QueryTransformer.build_filter(engine.dialect.name,
@@ -962,12 +1059,16 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 raise indexer.ResourceAttributeError(resource_type,
                                                      e.attribute)
 
-            q = q.filter(f)
+            q1 = session.query(target_cls.id)
+            q1 = q1.filter(f)
 
             session.query(Metric).filter(
-                Metric.resource_id.in_(q)
+                Metric.resource_id.in_(q1)
             ).update({"status": "delete"},
                      synchronize_session=False)
+
+            q = session.query(target_cls)
+            q = q.filter(f)
             return q.delete(synchronize_session=False)
 
     @retry_on_deadlock
