@@ -51,13 +51,17 @@ class ArchivePolicyDefinitionType(sqlalchemy_utils.JSONType):
     def process_result_value(self, value, dialect):
         values = super(ArchivePolicyDefinitionType,
                        self).process_result_value(value, dialect)
+        if values is None:
+            return []
         return [archive_policy.ArchivePolicyItem(**v) for v in values]
 
 
 class SetType(sqlalchemy_utils.JSONType):
     def process_result_value(self, value, dialect):
-        return set(super(SetType,
-                         self).process_result_value(value, dialect))
+        values = super(SetType, self).process_result_value(value, dialect)
+        if values is None:
+            return set()
+        return set(values)
 
 
 class ArchivePolicy(Base, GnocchiBase, archive_policy.ArchivePolicy):
@@ -198,6 +202,9 @@ class ResourceType(Base, GnocchiBase, resource_type.ResourceType):
 
 
 class ResourceJsonifier(indexer.Resource):
+    def __str__(self):
+        return str(self.jsonify())
+
     def jsonify(self, attrs=None):
         d = dict(self)
         del d['revision']
@@ -221,9 +228,14 @@ class ResourceJsonifier(indexer.Resource):
 class ResourceMixin(ResourceJsonifier):
     @declarative.declared_attr
     def __table_args__(cls):
-        return (sqlalchemy.CheckConstraint('started_at <= ended_at',
-                                           name="ck_started_before_ended"),
-                COMMON_TABLES_ARGS)
+        return (sqlalchemy.CheckConstraint(
+            'started_at <= ended_at',
+            name="ck_{}_started_before_ended".format(
+                cls.__tablename__
+            )
+        ),
+            COMMON_TABLES_ARGS
+        )
 
     @declarative.declared_attr
     def type(cls):
