@@ -17,6 +17,7 @@
 import collections
 import functools
 import itertools
+import logging
 import operator
 import uuid
 
@@ -42,6 +43,7 @@ from gnocchi import json
 from gnocchi import resource_type
 from gnocchi.rest.aggregates import exceptions
 from gnocchi.rest.aggregates import processor
+from gnocchi.rest import exceptions as rest_exceptions
 from gnocchi import storage
 from gnocchi import utils
 
@@ -54,6 +56,7 @@ except ImportError:
 
 
 ATTRGETTER_GRANULARITY = operator.attrgetter("granularity")
+LOG = logging.getLogger(__name__)
 
 
 def arg_to_list(value):
@@ -68,8 +71,16 @@ def abort(status_code, detail=''):
     """Like pecan.abort, but make sure detail is a string."""
     if status_code == 404 and not detail:
         raise RuntimeError("http code 404 must have 'detail' set")
-    if isinstance(detail, Exception):
+
+    if isinstance(detail, voluptuous.Invalid):
+        detail = {
+            'cause': 'Invalid input',
+            'reason': six.text_type(detail),
+            'detail': [six.text_type(path) for path in detail.path],
+        }
+    elif isinstance(detail, Exception):
         detail = detail.jsonify()
+    LOG.debug("Aborting request. Code [%s]. Details [%s]", status_code, detail)
     return pecan.abort(status_code, detail)
 
 
@@ -150,29 +161,25 @@ def deserialize(expected_content_types=None):
     try:
         params = json.load(pecan.request.body_file)
     except Exception as e:
-        abort(400, "Unable to decode body: " + six.text_type(e))
+        details = rest_exceptions.UnableToDecodeBody(e,
+                                                     pecan.request.body_file)
+        LOG.warning(details.jsonify())
+        abort(400, details)
     return params
 
 
-def validate(schema, data, required=True, detailed_exc=False):
+def validate(schema, data, required=True):
     try:
         return voluptuous.Schema(schema, required=required)(data)
     except voluptuous.Invalid as e:
-        if detailed_exc:
-            abort(400, {"cause": "Attribute value error",
-                        "reason": str(e),
-                        "detail": e.path})
-        else:
-            abort(400, "Invalid input: %s" % e)
+        abort(400, e)
 
 
 def deserialize_and_validate(schema, required=True,
-                             expected_content_types=None,
-                             detailed_exc=False):
+                             expected_content_types=None):
     return validate(schema,
                     deserialize(expected_content_types=expected_content_types),
-                    required,
-                    detailed_exc)
+                    required)
 
 
 def Timespan(value):
@@ -258,6 +265,8 @@ class ArchivePolicyController(rest.RestController):
 
         body = deserialize_and_validate(voluptuous.Schema({
             voluptuous.Required("definition"): ArchivePolicyDefinitionSchema,
+            voluptuous.Optional("back_window"): voluptuous.All(
+                voluptuous.Coerce(int), voluptuous.Range(min=0))
         }))
         # Validate the data
         try:
@@ -268,11 +277,12 @@ class ArchivePolicyController(rest.RestController):
 
         try:
             return pecan.request.indexer.update_archive_policy(
-                self.archive_policy, ap_items)
+                self.archive_policy, ap_items,
+                back_window=body.get('back_window'))
         except indexer.UnsupportedArchivePolicyChange as e:
             abort(400, six.text_type(e))
 
-    @pecan.expose()
+    @pecan.expose('json')
     def delete(self):
         # NOTE(jd) I don't think there's any point in fetching and passing the
         # archive policy here, as the rule is probably checking the actual role
@@ -353,7 +363,7 @@ class ArchivePolicyRulesController(rest.RestController):
             voluptuous.Required("name"): six.text_type,
             voluptuous.Required("metric_pattern"): six.text_type,
             voluptuous.Required("archive_policy_name"): six.text_type,
-            })
+        })
 
         body = deserialize_and_validate(ArchivePolicyRuleSchema)
         enforce("create archive policy rule", body)
@@ -391,7 +401,7 @@ class ArchivePolicyRuleController(rest.RestController):
     def patch(self):
         ArchivePolicyRuleSchema = voluptuous.Schema({
             voluptuous.Required("name"): six.text_type,
-            })
+        })
         body = deserialize_and_validate(ArchivePolicyRuleSchema)
         enforce("update archive policy rule", {})
         try:
@@ -400,7 +410,7 @@ class ArchivePolicyRuleController(rest.RestController):
         except indexer.UnsupportedArchivePolicyRuleChange as e:
             abort(400, six.text_type(e))
 
-    @pecan.expose()
+    @pecan.expose('json')
     def delete(self):
         # NOTE(jd) I don't think there's any point in fetching and passing the
         # archive policy rule here, as the rule is probably checking the actual
@@ -449,8 +459,7 @@ class MetricController(rest.RestController):
     @pecan.expose('json')
     def post_measures(self):
         self.enforce_metric("post measures")
-        measures = deserialize_and_validate(MeasuresListSchema,
-                                            detailed_exc=True)
+        measures = deserialize_and_validate(MeasuresListSchema)
         if measures:
             pecan.request.incoming.add_measures(self.metric.id, measures)
         pecan.response.status = 202
@@ -509,14 +518,19 @@ class MetricController(rest.RestController):
                 abort(503, 'Unable to refresh metric: %s. Metric is locked. '
                       'Please try again.' % self.metric.id)
         try:
-            return pecan.request.storage.get_measures(
-                self.metric, aggregations, start, stop, resample)[aggregation]
+            results = pecan.request.storage.get_aggregated_measures(
+                {self.metric: aggregations},
+                start, stop, resample)[self.metric]
+            return [(timestamp, results[key].aggregation.granularity, value)
+                    for key in sorted(results.keys(),
+                                      reverse=True)
+                    for timestamp, value in results[key]]
         except storage.AggregationDoesNotExist as e:
             abort(404, six.text_type(e))
         except storage.MetricDoesNotExist:
             return []
 
-    @pecan.expose()
+    @pecan.expose('json')
     def delete(self):
         self.enforce_metric("delete metric")
         try:
@@ -640,7 +654,10 @@ class MetricsController(rest.RestController):
     @classmethod
     @pecan.expose('json')
     def get_all(cls, **kwargs):
-        filtering = cls.MetricListSchema(kwargs)
+        try:
+            filtering = cls.MetricListSchema(kwargs)
+        except voluptuous.Error as e:
+            abort(400, "Invalid input: %s" % e)
 
         # Compat with old user/project API
         provided_user_id = filtering.pop('user_id', None)
@@ -890,25 +907,73 @@ class ResourceTypeController(rest.RestController):
                      if k not in rt_json_current["attributes"]}
         del_attrs = [k for k in rt_json_current["attributes"]
                      if k not in rt_json_next["attributes"]]
+        update_attrs = self.retrieve_update_attrs(rt_json_current,
+                                                  rt_json_next)
 
-        if not add_attrs and not del_attrs:
+        if update_attrs:
+            LOG.debug("Updating attributes [%s] for resource-type [%s]",
+                      update_attrs, self._name)
+
+        if not add_attrs and not del_attrs and not update_attrs:
             # NOTE(sileht): just returns the resource, the asked changes
             # just do nothing
             return rt
 
         try:
             add_attrs = schema.attributes_from_dict(add_attrs)
+            update_attrs = self.create_update_attrs(schema, update_attrs)
         except resource_type.InvalidResourceAttribute as e:
             abort(400, "Invalid input: %s" % e)
 
         try:
             return pecan.request.indexer.update_resource_type(
                 self._name, add_attributes=add_attrs,
-                del_attributes=del_attrs)
+                del_attributes=del_attrs, update_attributes=update_attrs)
         except indexer.NoSuchResourceType as e:
-                abort(400, six.text_type(e))
+            abort(400, six.text_type(e))
 
-    @pecan.expose()
+    def create_update_attrs(self, schema, update_attrs):
+        new_attrs = dict(map(lambda entry: (entry[0], entry[1][1]),
+                             update_attrs.items()))
+        old_attrs = dict(map(lambda entry: (entry[0], entry[1][0]),
+                             update_attrs.items()))
+        update_attrs_new = schema.attributes_from_dict(new_attrs)
+        update_attrs_new.sort(key=lambda attr: attr.name)
+        update_attrs_old = schema.attributes_from_dict(old_attrs)
+        update_attrs_old.sort(key=lambda attr: attr.name)
+        update_attrs = []
+        for i in range(len(update_attrs_new)):
+            update_attrs.append((update_attrs_new[i],
+                                 update_attrs_old[i]))
+
+        return update_attrs
+
+    def retrieve_update_attrs(self, rt_json_current, rt_json_next):
+        update_attrs = {}
+        for k, v in rt_json_current["attributes"].items():
+            if k in rt_json_next["attributes"]:
+                self.validate_types(k, rt_json_next, v)
+                should_be_updated = False
+                for kc, vc in v.items():
+                    if vc != rt_json_next["attributes"][k][kc]:
+                        should_be_updated = True
+                        break
+
+                if should_be_updated:
+                    update_attrs[k] = (v, rt_json_next["attributes"][k])
+
+        return update_attrs
+
+    def validate_types(self, attribute, new_json, old_json):
+        old_type = old_json['type']
+        new_type = new_json["attributes"][attribute]['type']
+        if new_type != old_type:
+            msg = "Type update is not available yet. Changing %s to %s " \
+                  "for attribute %s of resource %s" % (old_type, new_type,
+                                                       attribute, self._name)
+            abort(400, msg)
+
+    @pecan.expose('json')
     def delete(self):
         try:
             pecan.request.indexer.get_resource_type(self._name)
@@ -1011,16 +1076,11 @@ class ResourceController(rest.RestController):
             etag_set_headers(resource)
             return resource
 
-        for k, v in six.iteritems(body):
-            if k != 'metrics' and getattr(resource, k) != v:
-                create_revision = True
-                break
-        else:
-            if 'metrics' not in body:
-                # No need to go further, we assume the db resource
-                # doesn't change between the get and update
-                return resource
-            create_revision = False
+        create_revision = utils.is_resource_revision_needed(resource, body)
+        if not create_revision and 'metrics' not in body:
+            # No need to go further, we assume the db resource
+            # doesn't change between the get and update
+            return resource
 
         try:
             resource = pecan.request.indexer.update_resource(
@@ -1037,7 +1097,7 @@ class ResourceController(rest.RestController):
         etag_set_headers(resource)
         return resource
 
-    @pecan.expose()
+    @pecan.expose('json')
     def delete(self):
         resource = pecan.request.indexer.get_resource(
             self._resource_type, self.id)
@@ -1237,7 +1297,7 @@ class QueryStringSearchAttrFilter(object):
     @classmethod
     def _parsed_query2dict(cls, parsed_query):
         result = None
-        while parsed_query:
+        while len(parsed_query.asList()):
             part = parsed_query.pop()
             if part in cls.binary_operator:
                 result = {part: {parsed_query.pop(): result}}
@@ -1649,8 +1709,7 @@ class ResourcesMetricsMeasuresBatchController(rest.RestController):
         MeasuresBatchSchema = voluptuous.Schema(
             {functools.partial(ResourceID, creator=creator):
              {six.text_type: self.BackwardCompatibleMeasuresList}})
-        body = deserialize_and_validate(MeasuresBatchSchema,
-                                        detailed_exc=True)
+        body = deserialize_and_validate(MeasuresBatchSchema)
 
         known_metrics = []
         unknown_metrics = []
@@ -1839,6 +1898,7 @@ class AggregationResourceController(rest.RestController):
 
         return results
 
+
 FillSchema = voluptuous.Schema(
     voluptuous.Any(voluptuous.Coerce(float), "null", "dropna",
                    msg="Must be a float, 'dropna' or 'null'"))
@@ -2011,9 +2071,13 @@ class AggregationController(rest.RestController):
                         },
                     })
                 try:
-                    return pecan.request.storage.get_measures(
-                        metric, aggregations, start, stop, resample
-                    )[aggregation]
+                    results = pecan.request.storage.get_aggregated_measures(
+                        {metric: aggregations}, start, stop, resample)[metric]
+                    return [(timestamp, results[key].aggregation.granularity,
+                             value)
+                            for key in sorted(results.keys(),
+                                              reverse=True)
+                            for timestamp, value in results[key]]
                 except storage.MetricDoesNotExist:
                     return []
             return processor.get_measures(
@@ -2099,11 +2163,17 @@ class StatusController(rest.RestController):
                     metricd.MetricProcessor.GROUP_ID, member)
                 for member in members
             ]
-            report_dict['metricd']['processors'] = members
-            report_dict['metricd']['statistics'] = {
-                member: cap.get()
-                for member, cap in six.moves.zip(members, caps)
-            }
+            report_dict['metricd']['processors'] = [
+                member.decode() for member in members
+            ]
+            members_data = {}
+            for member, cap in six.moves.zip(members, caps):
+                caps_data = {
+                    six.ensure_str(k): v
+                    for k, v in six.iteritems(cap.get())
+                }
+                members_data[member.decode()] = caps_data
+            report_dict['metricd']['statistics'] = members_data
         else:
             report_dict['metricd']['processors'] = None
             report_dict['metricd']['statistics'] = {}
@@ -2226,7 +2296,7 @@ class PrometheusWriteController(rest.RestController):
         f.ParseFromString(buf)
         measures_by_rid = collections.defaultdict(dict)
         for ts in f.timeseries:
-            attrs = dict((l.name, l.value) for l in ts.labels)
+            attrs = dict((label.name, label.value) for label in ts.labels)
             original_rid = (attrs.get("job", "none"),
                             attrs.get("instance", "none"))
             name = attrs['__name__']
@@ -2234,7 +2304,7 @@ class PrometheusWriteController(rest.RestController):
                 data = [{'timestamp': s.timestamp_ms / 1000.0,
                          'value': s.value} for s in ts.samples]
                 measures_by_rid[original_rid][name] = validate(
-                    MeasuresListSchema, data, detailed_exc=True)
+                    MeasuresListSchema, data)
 
         creator = pecan.request.auth_helper.get_current_user(pecan.request)
 
@@ -2319,10 +2389,10 @@ class VersionsController(object):
                         {
                             "rel": "self",
                             "href": pecan.request.application_url + "/v1/"
-                            }
-                        ],
+                        }
+                    ],
                     "id": "v1.0",
                     "updated": "2015-03-19"
-                    }
-                ]
-            }
+                }
+            ]
+        }
